module SHABAL_TOP(
	clk, rst_n,                    /* clock and reset */
	init, EOM, load, fetch, idata, /* input from Control FPGA */
	ack, odata,                    /* output for Control FPGA */
	led                            /* LED for debug */
);   

/* Input/Output */
input         clk, rst_n;
input         init, EOM, load, fetch;
input  [15:0] idata;
output        ack;
output [15:0] odata;
output  [7:0] led;

/* register for debug */
reg debug0, debug1, debug2, debug3, debug4, debug5, debug6, debug7;

/* wire */
/* Shabal_INTERFACE -> Shabal_CORE */
wire        init_r, EOM_r, EN;
wire [31:0] idata32;

/* Shabal_CORE -> Shabal_INTERFACE */
wire        busy;
wire [31:0] hash0;
wire [31:0] hash1;
wire [31:0] hash2;
wire [31:0] hash3;
wire [31:0] hash4;
wire [31:0] hash5;
wire [31:0] hash6;
wire [31:0] hash7;

/* LED */
wire [7:0] led;

// LED for Debug (when msg = 
// 6162 6380 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000
// 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000)
//{{{/* Debug LED */
assign led[7:0] = {debug7,debug6,debug5,debug4,debug3,debug2,debug1,debug0};
always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug0 <= 1;
				else if (hash0 == 32'hAB5F2207) debug0 <= 0;
				else debug0 <= debug0;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug1 <= 1;
				else if (hash1 == 32'hFB48CA83) debug1 <= 0;
				else debug1 <= debug1;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug2 <= 1;
				else if (hash2 == 32'h21220D48) debug2 <= 0;
				else debug2 <= debug2;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug3 <= 1;
				else if (hash3 == 32'hCAD51094) debug3 <= 0;
				else debug3 <= debug3;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug4 <= 1;
				else if (hash4 == 32'hEF598300) debug4 <= 0;
				else debug4 <= debug4;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug5 <= 1;
				else if (hash5 == 32'h8215D3BF) debug5 <= 0;
				else debug5 <= debug5;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug6 <= 1;
				else if (hash6 == 32'hE2AF2990) debug6 <= 0;
				else debug6 <= debug6;
end

always @(posedge clk or negedge rst_n) begin
				if(~rst_n) debug7 <= 1;
				else if (hash7 == 32'h04043FCB) debug7 <= 0;
				else debug7 <= debug7;
end
//}}}
//}}}

/* instance */
SHABAL_INTERFACE shabal_interface(
   .clk(clk), .rst_n(rst_n),
   .init(init), .EOM(EOM), .load(load), .fetch(fetch), .idata(idata),
   .ack(ack), .odata(odata),
   .busy(busy), .hash0(hash0), .hash1(hash1), .hash2(hash2), .hash3(hash3), .hash4(hash4), .hash5(hash5), .hash6(hash6), .hash7(hash7),
   .init_r(init_r), .EOM_r(EOM_r), .EN(EN), .idata32(idata32)
);

SHABAL_CORE shabal_core(
   .clk(clk), .rst_n(rst_n),
   .init_r(init_r), .EOM_r(EOM_r), .EN(EN), .idata32(idata32),
   .busy(busy), .hash0(hash0), .hash1(hash1), .hash2(hash2), .hash3(hash3), .hash4(hash4), .hash5(hash5), .hash6(hash6), .hash7(hash7)
);

endmodule

// vim:fdm=marker
